﻿angular.module("fr.api", [])
	.constant("moment", moment)
	.service("api", ["$http", "$q", "$filter", "moment", function ($http, $q, $filter, moment) {
		var config = {
			api: "http://admin.footyroom.com/api/v2",
			content: "http://footyroom.com/mobile-app"
		};

		var cache = {},
			categoryRoot = null,
			self = this,
			matches = [];

		function clearCacheInGroup(cacheGroup) {
			if (typeof(cacheGroup) !== "string") {
				throw new Error("Must specify cacheGroup parameter.");
			}

			for (var property in cache) {
				if (cache.hasOwnProperty(property) && cache[property].group === cacheGroup) {
					delete cache[property];
				}
			}
		}

		function httpGet(uri, expiryInMilliseconds, clearCache, cacheGroup) {
			var deferred = $q.defer();
			var cacheItem = cache[uri];

			// If cache is available and is not expired.
			if (!clearCache && cacheItem != null && cacheItem.expiry > Date.now()) {
				deferred.resolve(cacheItem.value);
			} else {
				cache[uri] = {};
				// console.debug(uri);
				$http.get(uri, { cacheTimeout: 1000 * 10, retry: 10 }).success(function(result) {
					cache[uri] = {
						value: result,
						expiry: Date.now() + expiryInMilliseconds,
						group: cacheGroup
					};
					deferred.resolve(result);
				}).error(function() {
					deferred.reject();
				});
			}

			return deferred.promise;
		};

		function apiGet(uri, expiryInMilliseconds, clearCache, cacheGroup) {
			return httpGet(config.api + uri + "&api_key=f7d851bb79ac8a658e9c11d5e3c89f39", expiryInMilliseconds, clearCache, cacheGroup);
		}

		function getItem(getList, filter) {
			var deferred = $q.defer();

			getList().then(function(result) {
				var item = $filter('filter')(result, filter, true);

				if (item.length > 0) {
					deferred.resolve(item[0]);
				} else {
					deferred.resolve(null);
				}
			});

			return deferred.promise;
		}

		this.getCompetitions = function() {
			var deferred = $q.defer();
			var millisecondsInDay = 86400000;

			apiGet("/stages?hasPosts=1&type=competition", millisecondsInDay).then(function(stages) {
				if (stages[0].group === "latest") {
					deferred.resolve(stages);
					return;
				}

				stages = stages.filter(function(c) {
					// Hide Indian Super League (due to not working with them anymore).
					return c.id !== 2530;
				});

				categoryRoot = {
					id: 0,
					parent: null,
					name: "Latest games",
					group: null,
					sortRank: 0,
					tree: "",
					orderIndex: "/"
				};

				var groups = [];
				angular.forEach(stages, function(c) {
					var group = groups.find(function(g) {
						return g.name === c.group;
					});

					if (!group) {
						group = {
							id: c.id,
							name: c.group,
							parent: categoryRoot.id,
							level: "top",
							children: [],
							parentCategory: categoryRoot,
							sortRank: Number.MAX_VALUE
						};

						groups.push(group);
					}

					c.sortRank = c.sortRank || Number.MAX_VALUE;
					c.level = "node";
					c.parentCategory = group;
					c.children = [];

					group.children.push(c);
					group.sortRank = Math.min(group.sortRank, c.sortRank);
				});

				stages = stages.concat(groups);
				categoryRoot.children = groups;

				angular.forEach(stages, function(category) {
					var currentCategory = category;

					category.orderIndex = "";

					while (currentCategory != null && currentCategory !== categoryRoot) {
						category.orderIndex = pad(currentCategory.sortRank, 3) + "." + pad(currentCategory.id, 4) + "/" + category.orderIndex;

						currentCategory = currentCategory.parentCategory;
					}
				});

				// Prepend categoryRoot to the array.
				stages.unshift(categoryRoot);
				deferred.resolve(stages);
			});

			return deferred.promise;
		};

		this.getCompetition = function(filter) {
			return getItem(self.getCompetitions, filter);
		};

		this.getMatches = function(tree, page, clearCache) {
			var isLatestMatches = tree === "";

			if (clearCache) {
				// When clearing cache for matches, we need to clear cache for all pages for this particular tree.
				// This is necessary because we want to ensure that we don't clear cache just for the 1st, 2nd, etc page,
				// but rather for all pages. Otherwise we can end up in a situation where we've got latest version of the 1st page
				// but still looking at the old version of 2nd page, which can result in some data not being shown to the user.
				clearCacheInGroup(tree);
			}

			// For the "latest matches" we only want to show items that have posts, 
			// otherwise there will be lots of items without a video. For actual leagues
			// we want to show everything (i.e. - "hasPosts=0").
			var promise = apiGet("/matches?" + (isLatestMatches ? "has_post=1&" : "") + "status_type=finished&page=" + (page || 1) + "&stage_tree=" + tree, 1000 * 60 * 3, clearCache, tree);

			promise.then(function(result) {
				// Keep local cache of all matches, so that they can easily be found by `match_id` later.
				angular.forEach(result.data, function(match) {
					matches[match.match_id] = match;

					// Display all times in UTC, rather than in user's local time.
					var matchDate = moment(match.datetime.replace(" ", "T") + "Z").utc();
					match.date = matchDate.utc().format("LL");

					// Get stage tree for the season.
					match.seasonTree = match.stage_tree.substr(0, match.stage_tree.lastIndexOf(","));

					match.title = match.post != null ? match.post.post_title : (match.home_team.team_name + " " + match.home_score + " - " + match.away_score + " " + match.away_team.team_name);

					match.isEpl = match.stage_tree.indexOf("5,") === 0;

					// Season is always the second item in the tree. We need both competition
					// and season.
					match.seasonStage = match.stage_tree.split(",").slice(0, 2).join(",");
				});
			});

			return promise;
		};

		this.getMatch = function(matchId) {
			// Use promises (even though we resolve immediately), because in future we will probably
			// be retrieving by making an HTTP request.
			var deferred = $q.defer();
			deferred.resolve(matches[matchId]);
			return deferred.promise;
		};

		this.getLatestSeason = function(tree, includeAllStages) {
			var deferred = $q.defer();

			var competitionStageId = tree.indexOf(",") > -1 ? tree.substr(0, tree.indexOf(",")) : tree;

			apiGet("/stages?latestSeason=1&tree=" + competitionStageId, 1000 * 60 * 60 * 24).then(function(stages) {
				if (!includeAllStages) {
					var latestSeason = $filter("filter")(stages, { type: "season" }, true)[0];
					deferred.resolve(latestSeason);
				} else {
					deferred.resolve(stages);
				}
			});

			return deferred.promise;
		};

		this.getTables = function(tree, clearCache) {
			var deferred = $q.defer();

			if (typeof(tree) !== "string" || tree.length === 0) {
				deferred.resolve(null);
			} else {
				var promises = {
					stages: apiGet("/stages?tree=" + tree, 1000 * 60 * 60 * 24),
					tables: apiGet("/tables?stageTree=" + tree, 1000 * 60 * 10, clearCache)
				};

				$q.all(promises).then(function(results) {
					var tables = results.tables;
					var stages = results.stages;

					angular.forEach(tables, function(table) {
						var stage = $filter('filter')(stages, { tree: table.stageTree }, true)[0];
						table.stage = stage;
					});

					deferred.resolve(tables);
				});
			}

			return deferred.promise;
		};
	}]);